/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Dial plan macro wrapper to GoSub
 *
 * \author Atis Lezdins <atis@iq-labs.net>
 * 
 * \ingroup applications
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 0 $")

#include "asterisk/file.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/config.h"
#include "asterisk/utils.h"
#include "asterisk/lock.h"

static char *descrip =
"  Macro(macroname,arg1,arg2...): Executes GoSub call to allow use macros \n"
" written in AEL from realtime or extensions.conf \n";

static char *if_descrip =
"  MacroIf(<expr>?macroname_a[,arg1][:macroname_b[,arg1]])\n"
"Executes macro defined in <macroname_a> if <expr> is true\n"
"(otherwise <macroname_b> if provided)\n"
"Arguments as in application Macro()\n";

static char *app = "Macro";
static char *if_app = "MacroIf";

static char *synopsis = "Macro Implementation";
static char *if_synopsis = "Conditional Macro Implementation";

static int macro_exec(struct ast_channel *chan, void *data)
{
	char *tmp;
	char *rest;
	char *macro;
	int res=0;
	char gosubargs[255]="";

	struct ast_app *a = pbx_findapp("GoSub");

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "Macro() requires arguments. See \"core show application macro\" for help.\n");
		return -1;
	}

	tmp = ast_strdupa(data);
	rest = tmp;
	macro = strsep(&rest, ",");
	if (ast_strlen_zero(macro)) {
		ast_log(LOG_WARNING, "Invalid macro name specified\n");
		return 0;
	}

	if (!ast_exists_extension(chan, macro, "s", 1, chan->cid.cid_num)) {
		if (!ast_context_find(macro)) 
			ast_log(LOG_WARNING, "No such context '%s'\n", macro);
		else
			ast_log(LOG_WARNING, "Context '%s' lacks 's' extension, priority 1\n", macro);
		return 0;
	}

	if (!ast_strlen_zero(rest)) {
		snprintf(gosubargs,sizeof(gosubargs),"%s,s,1(%s)",macro,rest);
	} else {
		snprintf(gosubargs,sizeof(gosubargs),"%s,s,1",macro);
	}

	ast_log(LOG_NOTICE, "Executing GoSub(%s) \n",gosubargs);
	
	res = pbx_exec(chan,a,gosubargs);

	return res;
}

static int macroif_exec(struct ast_channel *chan, void *data) 
{
	char *expr = NULL, *label_a = NULL, *label_b = NULL;
	int res = 0;

	if (!(expr = ast_strdupa(data)))
		return -1;

	if ((label_a = strchr(expr, '?'))) {
		*label_a = '\0';
		label_a++;
		if ((label_b = strchr(label_a, ':'))) {
			*label_b = '\0';
			label_b++;
		}
		if (pbx_checkcondition(expr))
			res = macro_exec(chan, label_a);
		else if (label_b) 
			res = macro_exec(chan, label_b);
	} else
		ast_log(LOG_WARNING, "Invalid Syntax.\n");

	return res;
}
			
static int unload_module(void)
{
	int res;

	res = ast_unregister_application(if_app);
	res |= ast_unregister_application(app);

	return res;
}

static int load_module(void)
{
	int res;

	res |= ast_register_application(if_app, macroif_exec, if_synopsis, if_descrip);
	res |= ast_register_application(app, macro_exec, synopsis, descrip);

	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Extension Macros wrapper");
